%
% Reset all of the RNG's so that we can reproduce our results.
%
clear;
rand('seed',0);
randn('seed',0);
%
% Setup the problem data, initial conditions, and input.
%
deltat=0.01;                  % 0.01 second time steps.
nsteps=5000;                  % We'll go for 5000 time steps (50 seconds);
t=0.0:deltat:(deltat*(nsteps-1));
%
% A is built up by using Euler's method to discretize the
% differential equation.
%
A=eye(2)+deltat*[0 1; -1 -0.01];
%
% B is just the identity matrix scaled by deltat.
%
B=eye(2)*deltat;
%
% The initial condition.
%
x0=[0.1; 0.5];                % The true initial condition.
x0hat=[0.0; 0.0];             % Our guess.
P0hat=[0.5 0; 0 0.5];       % covariance of x0hat.
%
% Covariance for w.
%
Q=[0.0005 0; 0 0.0005];
%
% Setup the H matrix.  In this case, we assume that we observe
% x1(k) only and don't get to see x2(k).
%
H=[1 0];
%
% Covariance for v.
%
R=[0.0005];
%
% setup the input.
%

u=zeros(2,nsteps);
for k=1:nsteps
  u(2,k)=sin(2*t(k));
end
%
% Run the model forward in time to get the "true" results. 
%
xtrue=zeros(2,nsteps);
xtrue(:,1)=x0;
for k=2:nsteps
 xtrue(:,k)=A*xtrue(:,k-1)+B*u(:,k-1)+simmvn(zeros(2,1),Q);
end
%
% Plot the true solution.
%
figure(1);
clf;
subplot(2,1,1);
plot(t,xtrue(1,:),'k');
xlabel('t');
ylabel('x_1(t)');
axis([0 50 -2.0 2.0]);
subplot(2,1,2);
plot(t,xtrue(2,:),'k');
xlabel('t');
ylabel('x_2(t)');
axis([0 50 -2.0 2.0]);
print -deps kalmanfig1.eps
%
% Using prediciton steps only predict the state and find error bars.
%
xpred=zeros(2,nsteps);
uncert=zeros(2,nsteps);
xpred(:,1)=x0hat;
uncert(:,1)=sqrt(diag(P0hat));
Phat=P0hat;
for k=2:nsteps
 [xpred(:,k),Phat]=predict(A,B,u(:,k-1),xpred(:,k-1),Phat,Q);
 uncert(:,k)=sqrt(diag(Phat));
end
%
% Plot the predicted solution with error bars.
%
figure(2);
clf;
subplot(2,1,1);
plot(t,xpred(1,:),'k');
hold on
plot(t,xpred(1,:)+uncert(1,:),'k:');
plot(t,xpred(1,:)-uncert(1,:),'k:');
xlabel('t');
ylabel('xpred_1(t)');
axis([0 50 -2.0 2.0]);
subplot(2,1,2);
plot(t,xpred(2,:),'k');
hold on
plot(t,xpred(2,:)+uncert(2,:),'k:');
plot(t,xpred(2,:)-uncert(2,:),'k:');
xlabel('t');
ylabel('xpred_2(t)');
axis([0 50 -2.0 2.0]);
print -deps kalmanfig2.eps
%
% Using prediction and update steps, predict the state.
% We'll get observations every 1 seconds (100 time steps.)  
%
xpredk=zeros(2,nsteps);
uncertk=zeros(2,nsteps);
xpredk(:,1)=x0hat;
uncertk(:,1)=sqrt(diag(P0hat));
Phat=P0hat;
obscount=0;
for k=2:nsteps
%
% First, the prediction step.
%
 [xpredk(:,k),Phat]=predict(A,B,u(:,k-1),xpredk(:,k-1),Phat,Q);
%
% Every now and then, we get an observation.
%
 if (mod(k,100)==0)
   z=H*xtrue(:,k)+simmvn(zeros(1,1),R);
   [xpredk(:,k),Phat]=update(xpredk(:,k),Phat,z,H,R);
%
% Record the observation for plotting later.
%
   obscount=obscount+1;
   obsx1(obscount)=z;
   obstimes(obscount)=t(k);
 end
%
% Either way, update the error bar.
%
 uncertk(:,k)=sqrt(diag(Phat));
end
%
% Plot the predicted solution with error bars.
%
figure(3);
clf;
subplot(2,1,1);
plot(t,xpredk(1,:),'k');
hold on
plot(t,xpredk(1,:)+uncertk(1,:),'k:');
plot(t,xpredk(1,:)-uncertk(1,:),'k:');
%
% Plot the observations too.
%
plot(obstimes,obsx1,'ko');
%
% Add labels and adjust the axis.
%
xlabel('t');
ylabel('xpredk_1(t)');
axis([0 50 -2.0 2.0]);
subplot(2,1,2);
plot(t,xpredk(2,:),'k');
hold on
plot(t,xpredk(2,:)+uncertk(2,:),'k:');
plot(t,xpredk(2,:)-uncertk(2,:),'k:');
xlabel('t');
ylabel('xpredk_2(t)');
axis([0 50 -2.0 2.0]);
print -deps kalmanfig3.eps
%
% Next, the difference between xtrue and xpred.
%
figure(4);
clf;
subplot(2,1,1);
plot(t,xpred(1,:)-xtrue(1,:),'k');
%
% Add labels and adjust the axis.
%
xlabel('t');
ylabel('xpred_1(t)-x_1(t)');
axis([0 50 -1.0 1.0]);
subplot(2,1,2);
plot(t,xpred(2,:)-xtrue(2,:),'k');
xlabel('t');
ylabel('xpred_2(t)-x_2(t)');
axis([0 50 -1.0 1.0]);
print -deps kalmanfig4.eps
%
% Finally, the difference between xtrue and xpredk.
%
figure(5);
clf;
subplot(2,1,1);
plot(t,xpredk(1,:)-xtrue(1,:),'k');
%
% Add labels and adjust the axis.
%
xlabel('t');
ylabel('xpredk_1(t)-x_1(t)');
axis([0 50 -1.0 1.0]);
subplot(2,1,2);
plot(t,xpredk(2,:)-xtrue(2,:),'k');
xlabel('t');
ylabel('xpredk_2(t)-x_2(t)');
axis([0 50 -1.0 1.0]);
print -deps kalmanfig5.eps